import { Injectable } from '@angular/core';
import { Agent } from '../agent.service';

@Injectable()
export class Resource{
    protected type: string;
    protected id: number;

    constructor(protected req: Agent, type: string, id: number = null){
        this.type = type;
        this.id = id;
    }
    
    columns(callback: (response) => void) { 
        this.req.aGet(`${this.req.urls.base}/${this.type}/columns`, (response) => {
            callback(response);
        });        
    }
    
    rows(callback: (response) => void, payload?: Object) {
        this.req.aGet(`${this.req.urls.base}/${this.type}?payload=${JSON.stringify(payload)}`, (response) => {
            callback(response);
        });    
    }
    
    row(callback: (response) => void) {    
        this.req.aGet(`${this.req.urls.base}/${this.type}/${this.id}`, (response) => {
            callback(response);
        });    
    }
    
    create(payload: Object, callback: (response) => void) { 
        this.req.aPost(`${this.req.urls.base}/${this.type}`, payload, (response) => {
            callback(response);
        });        
    }
        
    update(payload: Object, callback: (response) => void) {
        this.req.aPost(`${this.req.urls.base}/${this.type}/${this.id}`, payload, (response) => {
            callback(response);
        });         
    }
    
    delete(callback: (response) => void) {
        this.req.aDelete(`${this.req.urls.base}/${this.type}/${this.id}`, (response) => {
            callback(response);
        });         
    }
}