import { Injectable } from '@angular/core';
import { Agent } from "../agent";
import { StorageService } from "../storage";
import { Md5 } from 'ts-md5/dist/md5';
import pluralize from 'pluralize';
import * as moment from 'moment';

/**
 * Tracks page activity and store locally,
 * transmit to server on subsequent visit
 */
@Injectable()
export class ActivitySpyService{
    private sessionId: string;
    private store: StorageService;

    public constructor(private agent: Agent, private storage: StorageService){
        this.store = this.storage.local();
    }

    /**
     * should be called only once per session
     */
    activate(){
        this.sessionId = this.store.get('sessionId');

        /**
         * if a session log has been created before
         * send to server, then empty
         * and prepare the store for fresh log
         */
        if(this.sessionId){
            this.agent.go((agent) => {
                //send log to server
                //one model at a time
                // let logs: Object[] = JSON.parse(this.store.get(this.sessionId));
                // logs.forEach((log, index, logs) => {
                //     agent.resource(pluralize(log['model'])).create(log['data'], (response) => {
                //         //remove log from local store
                //         logs.splice(index, 1);
                //     });
                // });
            });
        }
        
        //create new session
        //and prepare store
        this.agent.go((agent) => {
            // agent.guest().create({
            //     niche_id: 1,
            //     hash_id: Md5.hashStr(navigator.platform + navigator.userAgent + navigator.appCodeName + navigator.appName)
            // }, (response) => {
            //     this.sessionId = response.guest.id;
            //     this.store.set('sessionId', this.sessionId);
            //     this.store.set(this.sessionId, JSON.stringify([])); 
            // });  
        });
    }

    /**
     * called in response to events
     * 
     * add new log event to store
     * to be sent to server later
     * 
     * activity: click, hover, scroll, type (search)
     * element: anchor, page, button
     * detail: url, title 
     */
    detect(activity: string, element: string, detail: string){
        this.log('activity', {
            // dateTime: moment().format(),
            event: activity,
            element: element,
            value: detail
        });
    }

    /**
     * non-activity based events
     */
    session(){
        this.log('activity', {});
    }
    user(){
        this.log('activity', {
            // id: datum['id'],
            // proprietorId: datum['proprietor_id'],
            // guestHashId: datum['guest_hash_id'],
            // name: datum['name'],
            // email: datum['email'],
            // role: datum['role'],
            // avatar: { title: datum['name'] + '/' + datum['avatar'], url: datum['avatar'] },
            // summary: datum['summary'],
            // detail: datum['detail'],
            // status: datum['status'],
            // createdAt: moment(datum['created_at']).format(),
            // updatedAt: moment(datum['updated_at']).format()  
        });
    }
    page(){
        this.log('activity', {
            // id: datum['id'],
            // guestHashId: datum['guest_hash_id'],
            // startTime: datum['start_time'],
            // endTime: datum['start_time'],
            // status: datum['status'],
            // createdAt: moment(datum['created_at']).format(),
            // updatedAt: moment(datum['updated_at']).format()   
        });
    }
    share(){
        this.log('activity', {
            // id: datum['id'],
            // guestHashId: datum['guest_hash_id'],
            // stuffId: datum['stuff_id'],
            // key: datum['key'],
            // value: datum['value'],
            // status: datum['status'],
            // createdAt: moment(datum['created_at']).format(),
            // updatedAt: moment(datum['updated_at']).format()   
        });
    }
    track(){
        this.log('activity', {
            // id: datum['id'],
            // guestHashId: datum['guest_hash_id'],
            // stuffId: datum['stuff_id'],
            // key: datum['key'],
            // value: datum['value'],
            // status: datum['status'],
            // createdAt: moment(datum['created_at']).format(),
            // updatedAt: moment(datum['updated_at']).format()   
        });
    }

    log(model: string, data: Object){
        let log: Object[] = JSON.parse(this.store.get(this.sessionId));
        log.push({
            model: model,
            data: data
        });
        this.store.set(this.sessionId, JSON.stringify(log));
    }
}
